@@ -10,21 +10,23 @@ Copyright (c) 2021, David Hoffman
 import logging
 import os
 import subprocess
+import time
+from functools import partial
 
 import numpy as np
 import scipy
+import scipy.signal
 from numpy.fft import ifftshift, irfftn, rfftn
 from scipy.fft import next_fast_len
 from scipy.ndimage._ni_support import _normalize_sequence
 from scipy.ndimage.fourier import fourier_gaussian
-import scipy.signal
 
 logger = logging.getLogger(__name__)
 
 eps = np.finfo(float).eps
 
 
-def get_git(path="."):
+def get_git(path="."):  # pragma: no cover
     """Get git description."""
     try:
         # we slice to remove trailing new line.
@@ -53,6 +55,18 @@ def bin_ndarray(ndarray, new_shape=None, bin_size=None, operation="sum"):
     Returns
     -------
     binned array.
+
+    Example
+    -------
+    >>> a = np.arange(16).reshape(4, 4)
+    >>> a
+    array([[ 0,  1,  2,  3],
+           [ 4,  5,  6,  7],
+           [ 8,  9, 10, 11],
+           [12, 13, 14, 15]])
+    >>> bin_ndarray(a, bin_size=2)
+    array([[10, 18],
+           [42, 50]])
     """
     if new_shape is None:
         # if new shape isn't passed then calculate it
@@ -125,9 +139,8 @@ def scale(data, dtype=None):
     return ((data - dmin) / (dmax - dmin) * (tmax - tmin) + tmin).astype(dtype)
 
 
-def scale_uint16(data):
-    """Scale data to the uint16 range."""
-    return scale(data, np.uint16)
+scale_uint8 = partial(scale, dtype="uint8")
+scale_uint16 = partial(scale, dtype="uint16")
 
 
 def radial_profile(data, center=None, binsize=1.0):
@@ -199,7 +212,7 @@ def radial_profile(data, center=None, binsize=1.0):
     return radial_mean, radial_std
 
 
-def mode(data):
+def mode(data: np.ndarray) -> int:
     """Get mode of non-negative integer data.
 
     up to 1000 times faster than scipy mode
@@ -207,6 +220,22 @@ def mode(data):
 
     Note: we can vectorize this to work on different
     axes with numba
+
+    Parameters
+    ----------
+    data : np.ndarray
+        Data to get mode of
+
+    Returns
+    -------
+    mode : int
+        Modal value
+
+    Example
+    -------
+    >>> a = np.array([0, 0, 0, 1, 2, 3, 4, 4, 4, 4, 10])
+    >>> mode(a)
+    4
     """
     # will not work with negative numbers (for now)
     return np.bincount(data.ravel()).argmax()
@@ -279,15 +308,15 @@ def fft_pad(array, newshape=None, mode="median", **kwargs):
             newshape = tuple(newshape)
         elif isinstance(newshape, int) or np.issubdtype(newshape, np.integer):
             # test for regular python int, then numpy ints
-            newshape = tuple(newshape for n in oldshape)
+            newshape = tuple(newshape for _ in oldshape)
         else:
             raise ValueError(f"{newshape} is not a recognized shape")
     # generate padding and slices
-    padding, slices = padding_slices(oldshape, newshape)
+    padding, slices = _padding_slices(oldshape, newshape)
     return np.pad(array[slices], padding, mode=mode, **kwargs)
 
 
-def padding_slices(oldshape, newshape):
+def _padding_slices(oldshape, newshape):
     """Calculate the required padding or cropping based on the old shape and the new shape.
 
     Can be used to generate the slices needed to undo fft_pad above
@@ -482,7 +511,13 @@ def fft_gaussian_filter(img, sigma):
 
 
 def find_prime_facs(n):
-    """Find the prime factors of n."""
+    """Find the prime factors of n.
+    
+    Example
+    -------
+    >>> find_prime_facs(10)
+    array([2, 5])
+    """
     list_of_factors = []
     i = 2
     while n > 1:
@@ -584,3 +619,127 @@ def edf(stack):
     gradient_y = scipy.ndimage.sobel(img, 1)
     grad_img = gradient_x * gradient_x + gradient_y * gradient_y
     return get_max(stack, grad_img)
+
+
+def plane_fit(X, Y, Z):
+    """Fit a plane to data.
+    
+    Parameters
+    ----------
+    X : np.ndarray
+    Y : np.ndarray
+    Z : np.ndarray
+
+    Returns
+    -------
+    C : np.ndarray
+        Coefficients for plane fit
+    """
+    # build A matrix for plane
+    A = np.c_[X, Y, np.ones(len(Z))]
+
+    # calculate best fit coefs
+    C, _, _, _ = scipy.linalg.lstsq(A, Z)  # coefficients
+    return C
+
+
+def remove_tilt(X, Y, Z):
+    """Fit a plane to data and remove tilt (no rotation)."""
+    # build A matrix for plane
+    C = plane_fit(X, Y, Z)
+    return Z - C[0] * X - C[1] * Y
+
+
+def find_normal(X, Y, Z):
+    """Find the normal vector for a plane."""
+    C = plane_fit(X, Y, Z)
+    # calculate normal vector
+    normal = np.array((-C[0], -C[1], 1))
+
+    # return normalized normal vector
+    return normal / np.sqrt((normal ** 2).sum())
+
+
+def rot_matrix(source, target):
+    """Calculate the rotation matrix to rotate source to target."""
+    # https://math.stackexchange.com/questions/180418/calculate-rotation-matrix-to-align-vector-a-to-vector-b-in-3d
+    v1, v2, v3 = np.cross(source, target)
+    c = np.inner(source, target)
+    vx = np.array(((0, -v3, v2), (v3, 0, -v1), (-v2, v1, 0)))
+    return np.eye(3) + vx + vx @ vx * 1 / (1 + c)
+
+
+def calc_angles(mat_b):
+    """Calculate angles based on rotation matrix."""
+    atan2 = np.arctan2
+    return (
+        atan2(mat_b[1, 2], mat_b[2, 2]),
+        atan2(-mat_b[2, 0], np.sqrt(mat_b[1, 2] ** 2 + mat_b[2, 2] ** 2)),
+        atan2(mat_b[0, 1], mat_b[0, 0]),
+    )
+
+
+# TODO: refactor the below as methods of a point cloud object, maybe
+
+
+def fit_quadratic(x: np.ndarray, y: np.ndarray, z: np.ndarray) -> np.ndarray:
+    """Fit quadratic to point data."""
+    # assume data is on a regularly spaced grid
+
+    # flatten everything, just to be sure
+    X = x.ravel()
+    Y = y.ravel()
+    Z = z.ravel()
+
+    # build A matrix for quadratic function
+    A = np.c_[X * X, Y * Y, X, Y, X * Y, np.ones(len(Z))]
+
+    # calculate best fit coefs
+    C, _, _, _ = scipy.linalg.lstsq(A, Z)
+
+    return C
+
+
+def find_center(x: np.ndarray, y: np.ndarray, z: np.ndarray) -> np.ndarray:
+    """Find center of parabola.
+    
+    https://en.wikipedia.org/wiki/Quadratic_function#Minimum/maximum
+    """
+    # get fit coefs
+    aa, bb, cc, dd, ee, _ = fit_quadratic(x, y, z)
+    denom = 4 * aa * bb - ee * ee
+
+    # check if an extreme point is found
+    assert denom > 0, "No extrema found"
+
+    # calculate the maximum or minimum
+    x_m = -(2 * bb * cc - dd * ee) / denom
+    y_m = -(2 * aa * dd - cc * ee) / denom
+
+    return (y_m, x_m)
+
+
+class EasyTimer(object):
+    """Makes timing things easy with a `with` statement."""
+
+    def __init__(self, msg=""):
+        """Create an EasyTimer, choose the emit message."""
+        self.msg = msg
+
+    def __enter__(self):
+        """Start the timer."""
+        self.start = time.perf_counter()
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        """Stop the timer and emit a well formatted message."""
+        self.stop = time.perf_counter()
+        elapsed = self.stop - self.start
+        if elapsed < 1e-3:
+            elapsed_str = f"{elapsed * 1e6:.1f} µs"
+        elif elapsed < 1:
+            elapsed_str = f"{elapsed * 1e3:.1f} ms"
+        else:
+            elapsed_str = f"{elapsed:.1f} s"
+        print(f"{self.msg}: Elapsed time {elapsed_str}")
+        return False
